/* feat.c -- common routines for working with "features" files
 *           (exons, repeats, annotations, underlays, etc.)
 *
 * $Revision: 1.5 $$Date: 2001/08/15 01:11:18 $
 */

#include "feat.h"
#include "util.h"

static bool is_isolated (char *buf, char *b, char *t);
static bool is_exon_punct (char *buf, char *p);

/* extract interval endpoints and rest of buffer */
bool feat_parse_line (char *buf, char *head, int *a1, char *mid, int *a2,
                      char **tail) {
	char *b = buf;
	char *h = head;
	int n;

	if (starts (skip_ws (buf), "#"))
		return 0;

	while (*b) {
		/* copy non-position chars to head */
		while (*b && !isdigit (*b))    /* looking for unsigned integers */
			*h++ = *b++;
		*h = '\0';

		/* scan for interval */
		if (sscanf (b, "%d%[ \t]%d%n", a1, mid, a2, &n) == 3) {
			*tail = b + n;
			if (is_isolated (buf, b, *tail))    /* ok, values are good */
				return 1;
		}

		/* try again */
		while (*b && !isspace (*b))    /* exon_punct not applicable here */
			*h++ = *b++;
		*h = '\0';
	}
	return 0;
}

/* check whether extracted numbers are parts of other words */
static bool is_isolated (char *buf, char *b, char *t) {
	if (b > buf) {
		if (!isspace (*(b-1)) && !is_exon_punct (buf, b-1))
			return 0;
	}
	if (!isspace (*t))    /* tail contains at least '\n' */
		return 0;

	return 1;
}

/* identify special exons-file punctuation */
static bool is_exon_punct (char *buf, char *p) {
	return (p == skip_ws (buf) &&
			(*p == '>' || *p == '<' || *p == '|' || *p == '+'));
}

/* try to determine filetype from input line */
void feat_guess_filetype (char *buf, char *filetype) {
	char *p = skip_ws (buf);

	if (*p == '#')
		return;
	else if (*p == '>' || *p == '<' || *p == '|') {
		fprintf (stderr, "This appears to be an exons file.\n");
		*filetype = 'e';
	}
	else if (starts (p, "%define") && strstr (p, "annotation")) {
		fprintf (stderr, "This appears to be an annotations file.\n");
		*filetype = 'a';
	}
	else if (starts (p, "%:repeats") || strstr (p, "Right") ||
			strstr (p, "Left")) {
		fprintf (stderr, "This appears to be a repeats file.\n");
		*filetype = 'r';
	}
}

/* reverse direction indicators in input line; TAIL ARRAY MUST NOT BE FULL */
 void feat_reverse_dir (char *head, char *tail, char filetype) {
	char *p = skip_ws (head);

	if (*p == '#')
		return;
	else if (filetype == 'e') {
		if (*p == '>')
			*p = '<';
		else if (*p == '<')
			*p = '>';
	}
	else if (filetype == 'r') {
		if ((p = strstr (tail, "Right"))) {
			memmove (p+4, p+5, strlen (p+5) + 1);
			memcpy (p, "Left", 4);
		}
		else if ((p = strstr (tail, "Left"))) {
			memmove (p+5, p+4, strlen (p+4) + 1);   /* need room here */
			memcpy (p, "Right", 5);
		}
	}
}

